﻿using FluentAssertions;
using Magify.Model;
using NUnit.Framework;

namespace Magify.Tests
{
    internal partial class MagifyClientTests
    {
        public class AttStatus : MagifyClientTestsPart
        {
            [Test]
            public void WhenMagifyClientInit_AttStatusMustBeNull()
            {
                // Arrange
                // Act
                using var magifyClient = CreateDefaultAndInit();

                // Assert
                magifyClient.AttAuthorizationStatus.Value.Should().BeNull();
            }

            [Test]
            [TestCase(true)]
            [TestCase(false)]
            public void WhenMagifyClientInit_AndSetAttStatus_AttStatusMustBeChanged(bool value)
            {
                // Arrange
                using var magifyClient = CreateDefaultAndInit();

                // Act
                magifyClient.SetAttStatus(value);

                // Assert
                magifyClient.AttAuthorizationStatus.Value.Should().Be(value);
            }

            [Test]
            public void WhenMagifyClientInit_AuthTokenPayloadMustNotContainsIsIdfaAvailable()
            {
                // Arrange
                // Act
                using var magifyClient = CreateWithDeviceAndInit(EditorDevice.GetDefaultIOSDevice());

                // Assert
                var tokenPayload = (AuthTokenPayload)GetServerApi(magifyClient).CreateAuthTokenRequestMessage().Payload;
                tokenPayload.IsIdfaAvailable.Should().Be(null);
            }

            [Test]
            [TestCase(true)]
            [TestCase(false)]
            public void WhenMagifyClientInit_AndSetAttStatus_AuthTokenPayloadMustContainsIsIdfaAvailable(bool value)
            {
                // Arrange
                using var magifyClient = CreateWithDeviceAndInit(EditorDevice.GetDefaultIOSDevice());

                // Act
                magifyClient.SetAttStatus(value);

                // Assert
                var tokenPayload = (AuthTokenPayload)GetServerApi(magifyClient).CreateAuthTokenRequestMessage().Payload;
                tokenPayload.IsIdfaAvailable.Should().Be(value);
            }
        }
    }
}